@@ -26,6 +26,7 @@
 
 #include <gedit/gedit-commands.h>
 #include <gedit/gedit-utils.h>
+#include <gedit/gedit-app.h>
 #include <glib/gi18n-lib.h>
 #include <gedit/gedit-debug.h>
 #include <gconf/gconf-client.h>
@@ -83,6 +84,10 @@ static void on_virtual_root_changed_cb   (GeditFileBrowserStore * model,
 static void on_filter_mode_changed_cb    (GeditFileBrowserStore * model,
                                           GParamSpec * param,
                                           GeditWindow * window);
+static void on_rename_cb		 (GeditFileBrowserStore * model,
+					  const gchar * olduri,
+					  const gchar * newuri,
+					  GeditWindow * window);
 static void on_filter_pattern_changed_cb (GeditFileBrowserWidget * widget,
                                           GParamSpec * param,
                                           GeditWindow * window);
@@ -158,10 +163,8 @@ restore_default_location (GeditFileBrowserPlugin * plugin,
 	gboolean bookmarks;
 	gboolean remote;
 	GConfClient * client;
-	GFile * file = NULL;
 
 	client = gconf_client_get_default ();
-
 	if (!client)
 		return;
 
@@ -187,20 +190,25 @@ restore_default_location (GeditFileBrowserPlugin * plugin,
 	                                NULL);
 
 	if (root != NULL && *root != '\0') {
+		GFile *file;
+
 		file = g_file_new_for_uri (root);
 
-		if (!remote && !g_file_is_native (file)) {
-		} else if (virtual_root != NULL && virtual_root != '\0') {
-			prepare_auto_root(data);
-			gedit_file_browser_widget_set_root_and_virtual_root (data->tree_widget, 
-			                                                     root,
-			                                                     virtual_root);
-		} else {
-			prepare_auto_root(data);
-			gedit_file_browser_widget_set_root (data->tree_widget,
-			                                    root,
-			                                    TRUE);
+		if (remote || g_file_is_native (file)) {
+			if (virtual_root != NULL && virtual_root != '\0') {
+				prepare_auto_root(data);
+				gedit_file_browser_widget_set_root_and_virtual_root (data->tree_widget, 
+					                                             root,
+					                                             virtual_root);
+			} else {
+				prepare_auto_root(data);
+				gedit_file_browser_widget_set_root (data->tree_widget,
+					                            root,
+					                            TRUE);
+			}
 		}
+
+		g_object_unref (file);
 	}
 
 	g_object_unref (client);
@@ -217,7 +225,6 @@ restore_filter (GeditFileBrowserPluginData * data)
 	gchar *pattern;
 
 	client = gconf_client_get_default ();
-	
 	if (!client)
 		return;
 
@@ -716,7 +723,7 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 			  window);			  
 
 	panel = gedit_window_get_side_panel (window);
-	pixbuf = gedit_file_browser_utils_pixbuf_from_theme("file-manager",  
+	pixbuf = gedit_file_browser_utils_pixbuf_from_theme("system-file-manager",  
 	                                                    GTK_ICON_SIZE_MENU);
 	
 	if (pixbuf) {
@@ -762,6 +769,11 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	                  G_CALLBACK (on_filter_mode_changed_cb),
 	                  window);
 
+	g_signal_connect (store,
+			  "rename",
+			  G_CALLBACK (on_rename_cb),
+			  window);
+
 	g_signal_connect (window,
 	                  "tab-added",
 	                  G_CALLBACK (on_tab_added_cb),
@@ -961,6 +973,68 @@ on_filter_mode_changed_cb (GeditFileBrowserStore * model,
 	
 }
 
+static void
+on_rename_cb (GeditFileBrowserStore * store,
+	      const gchar * olduri,
+	      const gchar * newuri,
+	      GeditWindow * window)
+{
+	GeditApp * app;
+	GList * documents;
+	GList * item;
+	GeditDocument * doc;
+	GFile * docfile;
+	GFile * oldfile;
+	GFile * newfile;
+	gchar * uri;
+	gchar * relative;
+	
+	/* Find all documents and set its uri to newuri where it matches olduri */
+	app = gedit_app_get_default ();
+	documents = gedit_app_get_documents (app);
+	
+	oldfile = g_file_new_for_uri (olduri);
+	newfile = g_file_new_for_uri (newuri);
+	
+	for (item = documents; item; item = item->next) {
+		doc = GEDIT_DOCUMENT (item->data);
+		uri = gedit_document_get_uri (doc);
+		
+		if (!uri)
+			continue;
+		
+		docfile = g_file_new_for_uri (uri);
+
+		if (g_file_equal (docfile, oldfile)) {
+			gedit_document_set_uri (doc, newuri);
+		} else {
+			relative = g_file_get_relative_path (oldfile, docfile);
+			g_object_unref (docfile);
+			g_free (uri);
+		
+			if (relative) {
+				/* relative now contains the part in docfile without
+				   the prefix oldfile */
+
+				docfile = g_file_get_child (newfile, relative);
+				uri = g_file_get_uri (docfile);
+			
+				gedit_document_set_uri (doc, uri);
+			}
+		
+			g_free (relative);
+		}
+		
+		g_free (uri);
+		g_object_unref (docfile);
+	}
+	
+	g_object_unref (oldfile);
+	g_object_unref (newfile);
+	
+	g_list_free (documents);
+}
+
 static void 
 on_filter_pattern_changed_cb (GeditFileBrowserWidget * widget,
                               GParamSpec * param,
