@@ -51,11 +51,13 @@
 				 G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN "," \
 			 	 G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP "," \
 				 G_FILE_ATTRIBUTE_STANDARD_NAME "," \
-				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
+				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," \
+				 G_FILE_ATTRIBUTE_STANDARD_ICON
 
 typedef struct _FileBrowserNode    FileBrowserNode;
 typedef struct _FileBrowserNodeDir FileBrowserNodeDir;
 typedef struct _AsyncData	   AsyncData;
+typedef struct _AsyncNode	   AsyncNode;
 
 typedef gint (*SortFunc) (FileBrowserNode * node1,
 			  FileBrowserNode * node2);
@@ -70,6 +72,12 @@ struct _AsyncData
 	gboolean removed;
 };
 
+struct _AsyncNode
+{
+	FileBrowserNodeDir *dir;
+	GCancellable *cancellable;
+};
+
 struct _FileBrowserNode 
 {
 	GFile *file;
@@ -173,19 +181,10 @@ static void model_clear                                     (GeditFileBrowserSto
 							     gboolean free_nodes);
 static gint model_sort_default                              (FileBrowserNode * node1,
 							     FileBrowserNode * node2);
-static void print_tree                                      (GeditFileBrowserStore * model,
-							     FileBrowserNode * parent, 
-							     gchar * prefix);
 static void model_check_dummy                               (GeditFileBrowserStore * model,
 							     FileBrowserNode * node);
 static void next_files_async 				    (GFileEnumerator * enumerator,
-							     FileBrowserNodeDir * dir);
-
-static void on_directory_monitor_event                      (GFileMonitor * handle,
-							     GFile *monitor_file,
-							     GFile * info_file,
-							     guint event_type,
-							     FileBrowserNode * parent);
+							     AsyncNode * async);
 
 GEDIT_PLUGIN_DEFINE_TYPE_WITH_CODE (GeditFileBrowserStore, gedit_file_browser_store,
 			G_TYPE_OBJECT,
@@ -1045,77 +1044,6 @@ model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 	return collate_nodes (node1, node2);
 }
 
-static void
-model_resort_children (GeditFileBrowserStore * model,
-		       FileBrowserNode * parent, GtkTreePath * path)
-{
-	FileBrowserNodeDir *dir;
-	FileBrowserNode *node;
-	GSList *item;
-	gint *neworder;
-	gint pos = 0;
-	gboolean visible;
-	GtkTreeIter iter;
-	gboolean free_path = FALSE;
-
-	if (!NODE_IS_DIR (parent))
-		return;
-
-	dir = FILE_BROWSER_NODE_DIR (parent);
-
-	if (!dir->children)
-		return;
-
-	visible = model_node_visibility (model, parent);
-
-	if (visible) {
-		pos = 0;
-
-		if (path == NULL) {
-			path = gtk_tree_path_new ();
-			free_path = TRUE;
-		}
-	}
-
-	gtk_tree_path_up (path);
-
-	for (item = dir->children; item; item = item->next) {
-		node = (FileBrowserNode *) (item->data);
-		model_resort_children (model, node, path);
-
-		if (visible && model_node_visibility (model, node)) {
-			node->pos = pos++;
-			gtk_tree_path_next (path);
-		}
-	}
-
-	gtk_tree_path_down (path);
-
-	if (visible) {
-		dir->children =
-		    g_slist_sort (dir->children,
-				  (GCompareFunc) (model->priv->sort_func));
-		neworder = g_new (gint, pos);
-
-		pos = 0;
-
-		for (item = dir->children; item; item = item->next) {
-			node = (FileBrowserNode *) (item->data);
-
-			if (model_node_visibility (model, node))
-				neworder[pos++] = node->pos;
-		}
-
-		iter.user_data = parent;
-		gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
-					       path, &iter, neworder);
-		g_free (neworder);
-
-		if (free_path)
-			gtk_tree_path_free (path);
-	}
-}
-
 static void
 model_resort_node (GeditFileBrowserStore * model, FileBrowserNode * node)
 {
@@ -1203,14 +1131,14 @@ row_inserted (GeditFileBrowserStore * model,
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL(model), copy, iter);
 	gtk_tree_path_free (copy);
 	
-	if (ref) {
+	if (ref)
+	{
 		gtk_tree_path_free (*path);
 
 		/* To restore the path, we get the path from the reference. But, since
 		   we inserted a row, the path will be one index further than the
 		   actual path of our node. We therefore call gtk_tree_path_prev */
 		*path = gtk_tree_row_reference_get_path (ref);
-	
 		gtk_tree_path_prev (*path);
 	}
 
@@ -1232,14 +1160,14 @@ row_deleted (GeditFileBrowserStore * model,
 static void
 model_refilter_node (GeditFileBrowserStore * model,
 		     FileBrowserNode * node,
-		     GtkTreePath * path)
+		     GtkTreePath ** path)
 {
 	gboolean old_visible;
 	gboolean new_visible;
 	FileBrowserNodeDir *dir;
 	GSList *item;
 	GtkTreeIter iter;
-	gboolean free_path = FALSE;
+	GtkTreePath *tmppath = NULL;
 	gboolean in_tree;
 
 	if (node == NULL)
@@ -1250,19 +1178,20 @@ model_refilter_node (GeditFileBrowserStore * model,
 
 	in_tree = node_in_tree (model, node);
 
-	if (path == NULL) {
+	if (path == NULL)
+	{
 		if (in_tree)
-			path = gedit_file_browser_store_get_path_real (model,
+			tmppath = gedit_file_browser_store_get_path_real (model,
 								       node);
 		else
-			path = gtk_tree_path_new_first ();
+			tmppath = gtk_tree_path_new_first ();
 
-		free_path = TRUE;
+		path = &tmppath;
 	}
 
 	if (NODE_IS_DIR (node)) {
 		if (in_tree)
-			gtk_tree_path_down (path);
+			gtk_tree_path_down (*path);
 
 		dir = FILE_BROWSER_NODE_DIR (node);
 
@@ -1273,7 +1202,7 @@ model_refilter_node (GeditFileBrowserStore * model,
 		}
 
 		if (in_tree)
-			gtk_tree_path_up (path);
+			gtk_tree_path_up (*path);
 	}
 
 	if (in_tree) {
@@ -1282,21 +1211,21 @@ model_refilter_node (GeditFileBrowserStore * model,
 		if (old_visible != new_visible) {
 			if (old_visible) {
 				node->inserted = FALSE;
-				row_deleted (model, path);
+				row_deleted (model, *path);
 			} else {
 				iter.user_data = node;
-				row_inserted (model, &path, &iter);
-				gtk_tree_path_next (path);
+				row_inserted (model, path, &iter);
+				gtk_tree_path_next (*path);
 			}
 		} else if (old_visible) {
-			gtk_tree_path_next (path);
+			gtk_tree_path_next (*path);
 		}
 	}
 	
 	model_check_dummy (model, node);
 
-	if (free_path)
-		gtk_tree_path_free (path);
+	if (tmppath)
+		gtk_tree_path_free (tmppath);
 }
 
 static void
@@ -1623,7 +1552,8 @@ file_browser_node_unload (GeditFileBrowserStore * model,
 
 static void
 model_recomposite_icon_real (GeditFileBrowserStore * tree_model,
-			     FileBrowserNode * node)
+			     FileBrowserNode * node,
+			     GFileInfo * info)
 {
 	GtkIconTheme *theme;
 	GdkPixbuf *icon;
@@ -1635,7 +1565,11 @@ model_recomposite_icon_real (GeditFileBrowserStore * tree_model,
 		return;
 
 	theme = gtk_icon_theme_get_default ();
-	icon = gedit_file_browser_utils_pixbuf_from_file (node->file, GTK_ICON_SIZE_MENU);
+	
+	if (info)
+		icon = gedit_file_browser_utils_pixbuf_from_icon (g_file_info_get_icon (info), GTK_ICON_SIZE_MENU);
+	else
+		icon = gedit_file_browser_utils_pixbuf_from_file (node->file, GTK_ICON_SIZE_MENU);
 
 	if (node->icon)
 		g_object_unref (node->icon);
@@ -1678,7 +1612,8 @@ model_recomposite_icon (GeditFileBrowserStore * tree_model,
 
 	model_recomposite_icon_real (tree_model,
 				     (FileBrowserNode *) (iter->
-							  user_data));
+							  user_data),
+				     NULL);
 }
 
 static FileBrowserNode *
@@ -1863,12 +1798,13 @@ file_browser_node_set_from_info (GeditFileBrowserStore * model,
 					  &error);
 					  
 		if (!info) {
-			uri = g_file_get_uri (node->file);
-			
-			g_warning ("Could not get info for %s: %s", uri, error->message);
+			if (!(error->domain == G_IO_ERROR && error->code == G_IO_ERROR_NOT_FOUND)) {
+				uri = g_file_get_uri (node->file);
+				g_warning ("Could not get info for %s: %s", uri, error->message);
+				g_free (uri);
+			}
 			g_error_free (error);
-			g_free (uri);
-			
+
 			return;
 		}
 		
@@ -1896,14 +1832,14 @@ file_browser_node_set_from_info (GeditFileBrowserStore * model,
 			node->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_IS_TEXT;		
 	}
 	
+	model_recomposite_icon_real (model, node, info);
+
 	if (free_info)
 		g_object_unref (info);
 
-	model_recomposite_icon_real (model, node);
-
 	if (isadded) {
 		path = gedit_file_browser_store_get_path_real (model, node);
-		model_refilter_node (model, node, path);
+		model_refilter_node (model, node, &path);
 		gtk_tree_path_free (path);
 		
 		model_check_dummy (model, node->parent);
@@ -1980,22 +1916,21 @@ model_add_node_from_file (GeditFileBrowserStore * model,
 static FileBrowserNode *
 model_add_node_from_dir (GeditFileBrowserStore * model,
 			 FileBrowserNode * parent,
-			 GFile * file,
-			 GFileInfo * info)
+			 GFile * file)
 {
 	FileBrowserNode *node;
 
-	// Check if it already exists
+	/* Check if it already exists */
 	if ((node = model_file_exists (model, parent, file)) == NULL) {	
 		node = file_browser_node_dir_new (model, file, parent);
-		file_browser_node_set_from_info (model, node, info, FALSE);
+		file_browser_node_set_from_info (model, node, NULL, FALSE);
 
 		if (node->name == NULL) {
 			file_browser_node_set_name (node);
 		}
-		
+
 		if (node->icon == NULL) {
-			node->icon = gedit_file_browser_utils_pixbuf_from_theme ("gnome-fs-directory", GTK_ICON_SIZE_MENU);
+			node->icon = gedit_file_browser_utils_pixbuf_from_theme ("folder", GTK_ICON_SIZE_MENU);
 		}
 
 		model_add_node (model, node, parent);
@@ -2076,10 +2011,47 @@ parse_dot_hidden_file (FileBrowserNode *directory)
 	g_free (file_contents);
 }
 
+static void
+on_directory_monitor_event (GFileMonitor * monitor,
+			    GFile * file,
+			    GFile * other_file,
+			    GFileMonitorEvent event_type,
+			    FileBrowserNode * parent)
+{
+	FileBrowserNode *node;
+	FileBrowserNodeDir *dir = FILE_BROWSER_NODE_DIR (parent);
+
+	switch (event_type) {
+	case G_FILE_MONITOR_EVENT_DELETED:
+		node = model_file_exists (dir->model, parent, file);
+
+		if (node != NULL) {
+			// Remove the node
+			model_remove_node (dir->model, node, NULL, TRUE);
+		}
+		break;
+	case G_FILE_MONITOR_EVENT_CREATED:
+		if (g_file_query_exists (file, NULL)) {
+			model_add_node_from_file (dir->model, parent, file, NULL);
+		}
+		
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+async_node_free (AsyncNode *async)
+{
+	g_object_unref (async->cancellable);
+	g_free (async);
+}
+
 static void
 model_iterate_next_files_cb (GFileEnumerator * enumerator, 
 			     GAsyncResult * result, 
-			     FileBrowserNode * parent)
+			     AsyncNode * async)
 {
 	GList * files;
 	GList * item;
@@ -2088,12 +2060,14 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 	GFileType type;
 	gchar const * name;
 	GFile * file;
-	FileBrowserNodeDir * dir = FILE_BROWSER_NODE_DIR (parent);
+	FileBrowserNodeDir * dir = async->dir;
+	FileBrowserNode * parent = (FileBrowserNode *)dir;
 	
 	files = g_file_enumerator_next_files_finish (enumerator, result, &error);
 
 	if (files == NULL) {
 		g_file_enumerator_close (enumerator, NULL, NULL);
+		async_node_free (async);
 		
 		if (!error)
 		{
@@ -2129,6 +2103,10 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 			file_browser_node_unload (dir->model, (FileBrowserNode *)parent, TRUE);
 			g_error_free (error);
 		}
+	} else if (g_cancellable_is_cancelled (async->cancellable)) {
+		/* Check cancel state manually */
+		g_file_enumerator_close (enumerator, NULL, NULL);
+		async_node_free (async);
 	} else {
 		for (item = files; item; item = item->next) {
 			info = G_FILE_INFO (item->data);
@@ -2156,36 +2134,41 @@ model_iterate_next_files_cb (GFileEnumerator * enumerator,
 		}
 		
 		g_list_free (files);
-		next_files_async (enumerator, dir);
+		next_files_async (enumerator, async);
 	}
 }
 
 static void
 next_files_async (GFileEnumerator * enumerator,
-		  FileBrowserNodeDir * dir)
+		  AsyncNode * async)
 {
 	g_file_enumerator_next_files_async (enumerator,
 					    DIRECTORY_LOAD_ITEMS_PER_CALLBACK,
 					    G_PRIORITY_DEFAULT,
-					    dir->cancellable,
+					    async->cancellable,
 					    (GAsyncReadyCallback)model_iterate_next_files_cb,
-					    dir);
+					    async);
 }
 
 static void
 model_iterate_children_cb (GFile * file, 
 			   GAsyncResult * result,
-			   FileBrowserNodeDir * dir)
+			   AsyncNode * async)
 {
 	GError * error = NULL;
 	GFileEnumerator * enumerator;
+
+	if (g_cancellable_is_cancelled (async->cancellable))
+	{
+		async_node_free (async);
+		return;
+	}
 	
 	enumerator = g_file_enumerate_children_finish (file, result, &error);
-	
+
 	if (enumerator == NULL) {
-		/* Simply return if we were cancelled */
-		if (error->domain == G_IO_ERROR && error->code == G_IO_ERROR_CANCELLED)
-			return;
+		/* Simply return if we were cancelled or if the dir is not there */
+		FileBrowserNodeDir *dir = async->dir;
 		
 		/* Otherwise handle the error appropriately */
 		g_signal_emit (dir->model,
@@ -2196,8 +2179,9 @@ model_iterate_children_cb (GFile * file,
 
 		file_browser_node_unload (dir->model, (FileBrowserNode *)dir, TRUE);
 		g_error_free (error);
+		async_node_free (async);
 	} else {
-		next_files_async (enumerator, dir);
+		next_files_async (enumerator, async);
 	}
 }
 
@@ -2223,15 +2207,19 @@ model_load_directory (GeditFileBrowserStore * model,
 	parse_dot_hidden_file (node);
 	
 	dir->cancellable = g_cancellable_new ();
+	
+	AsyncNode *async = g_new (AsyncNode, 1);
+	async->dir = dir;
+	async->cancellable = g_object_ref (dir->cancellable);
 
 	/* Start loading async */
 	g_file_enumerate_children_async (node->file,
 					 STANDARD_ATTRIBUTE_TYPES,
 					 G_FILE_QUERY_INFO_NONE,
 					 G_PRIORITY_DEFAULT,
-					 dir->cancellable,
+					 async->cancellable,
 					 (GAsyncReadyCallback)model_iterate_children_cb,
-					 dir);
+					 async);
 }
 
 static GList *
@@ -2314,40 +2302,6 @@ model_fill (GeditFileBrowserStore * model, FileBrowserNode * node,
 		gtk_tree_path_free (path);
 }
 
-static void
-print_tree (GeditFileBrowserStore * model, FileBrowserNode * parent,
-	    gchar * prefix)
-{
-	GSList *item;
-	gchar *newpref;
-
-	if (parent == NULL)
-		parent = model->priv->root;
-
-	if (parent == NULL)
-		return;
-
-	if (prefix == NULL)
-		prefix = "";
-
-	g_message ("%s - %s (%d), loaded: %d", prefix, parent->name,
-		   model_node_visibility (model, parent),
-		   NODE_LOADED (parent));
-
-	if (NODE_IS_DIR (parent)) {
-		newpref = g_strdup_printf ("\t%s", prefix);
-
-		for (item = FILE_BROWSER_NODE_DIR (parent)->children; item;
-		     item = item->next) {
-			print_tree (model,
-				    (FileBrowserNode *) (item->data),
-				    newpref);
-		}
-
-		g_free (newpref);
-	}
-}
-
 static void
 set_virtual_root_from_node (GeditFileBrowserStore * model,
 			    FileBrowserNode * node)
@@ -2449,7 +2403,7 @@ set_virtual_root_from_file (GeditFileBrowserStore * model,
 	for (item = files; item; item = item->next) {
 		check = G_FILE (item->data);
 		
-		parent = model_add_node_from_dir (model, parent, check, NULL);
+		parent = model_add_node_from_dir (model, parent, check);
 		g_object_unref (check);
 	}
 
@@ -2560,48 +2514,70 @@ typedef struct {
 	GeditFileBrowserStore * model;
 	gchar * virtual_root;
 	GMountOperation * operation;
+	GCancellable * cancellable;
 } MountInfo;
 
+
+static void
+handle_root_error (GeditFileBrowserStore * model, GError *error)
+{
+	FileBrowserNode * root;
+
+	g_signal_emit (model, 
+		       model_signals[ERROR], 
+		       0, 
+		       GEDIT_FILE_BROWSER_ERROR_SET_ROOT,
+		       error->message);
+	
+	/* Set the virtual root to the root */
+	root = model->priv->root;
+	model->priv->virtual_root = root;
+	
+	/* Set the root to be loaded */
+	root->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_LOADED;
+	
+	/* Check the dummy */
+	model_check_dummy (model, root);
+	
+	g_object_notify (G_OBJECT (model), "root");
+	g_object_notify (G_OBJECT (model), "virtual-root");
+}
+
 static void
 mount_cb (GFile * file, 
 	  GAsyncResult * res, 
 	  MountInfo * mount_info)
 {
-	GFile * mounted_on;
+	gboolean mounted;
 	GError * error = NULL;
-	FileBrowserNode * root;
 	GeditFileBrowserStore * model = mount_info->model;
 	
-	mounted_on = g_file_mount_mountable_finish (file, res, &error);
+	mounted = g_file_mount_enclosing_volume_finish (file, res, &error);
 	
-	if (mounted_on) {
+	if (g_cancellable_is_cancelled (mount_info->cancellable))
+	{
+		if (error)
+			g_error_free (error);
+		
+		// Reset because it might be reused?
+		g_cancellable_reset (mount_info->cancellable);
+	}
+	else if (mounted)
+	{
 		model_root_mounted (model, mount_info->virtual_root);
-		g_object_unref (mounted_on);
-	} else if (error->code != G_IO_ERROR_CANCELLED) {
-		g_signal_emit (model, 
-			       model_signals[ERROR], 
-			       0, 
-			       GEDIT_FILE_BROWSER_ERROR_SET_ROOT,
-			       error->message);
-
-		g_error_free (error);
-		
-		/* Set the virtual root to the root */
-		root = model->priv->root;
-		model->priv->virtual_root = root;
-		
-		/* Set the root to be loaded */
-		root->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_LOADED;
-		
-		/* Check the dummy */
-		model_check_dummy (model, root);
-		
-		g_object_notify (G_OBJECT (model), "root");
-		g_object_notify (G_OBJECT (model), "virtual-root");
+	}
+	else if (error->code != G_IO_ERROR_CANCELLED)
+	{
+		handle_root_error (model, error);
 	}
 	
+	if (error)
+		g_error_free (error);
+
 	g_object_unref (mount_info->operation);
+	g_object_unref (mount_info->cancellable);
 	g_free (mount_info->virtual_root);
+
 	g_free (mount_info);
 }
 
@@ -2626,15 +2602,21 @@ model_mount_root (GeditFileBrowserStore * model, gchar const * virtual_root)
 			mount_info = g_new(MountInfo, 1);
 			mount_info->model = model;
 			mount_info->virtual_root = g_strdup (virtual_root);
-			mount_info->operation = g_mount_operation_new ();
 			
-			g_file_mount_mountable (model->priv->root->file, 
-						G_MOUNT_MOUNT_NONE,
-						mount_info->operation,
-						FILE_BROWSER_NODE_DIR (model->priv->root)->cancellable,
-						(GAsyncReadyCallback)mount_cb,
-						mount_info);
-						
+			/* FIXME: we should be setting the correct window */
+			mount_info->operation = gtk_mount_operation_new (NULL);
+			mount_info->cancellable = g_object_ref (FILE_BROWSER_NODE_DIR (model->priv->root)->cancellable);
+			
+			g_file_mount_enclosing_volume (model->priv->root->file, 
+						       G_MOUNT_MOUNT_NONE,
+						       mount_info->operation,
+						       mount_info->cancellable,
+						       (GAsyncReadyCallback)mount_cb,
+						       mount_info);
+		}
+		else
+		{
+			handle_root_error (model, error);
 		}
 		
 		g_error_free (error);
@@ -3455,35 +3437,4 @@ gedit_file_browser_store_new_directory (GeditFileBrowserStore * model,
 	return result;
 }
 
-/* Signal handlers */
-static void
-on_directory_monitor_event (GFileMonitor * monitor,
-			    GFile * file,
-			    GFile * other_file,
-			    GFileMonitorEvent event_type,
-			    FileBrowserNode * parent)
-{
-	FileBrowserNode *node;
-	FileBrowserNodeDir *dir = FILE_BROWSER_NODE_DIR (parent);
-
-	switch (event_type) {
-	case G_FILE_MONITOR_EVENT_DELETED:
-		node = model_file_exists (dir->model, parent, file);
-
-		if (node != NULL) {
-			// Remove the node
-			model_remove_node (dir->model, node, NULL, TRUE);
-		}
-		break;
-	case G_FILE_MONITOR_EVENT_CREATED:
-		if (g_file_query_exists (file, NULL)) {
-			model_add_node_from_file (dir->model, parent, file, NULL);
-		}
-		
-		break;
-	default:
-		break;
-	}
-}
-
 // ex:ts=8:noet:
