@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gedit-time-plugin.c 6028 2007-12-22 16:07:19Z pborelli $
+ * $Id: gedit-time-plugin.c 6379 2008-08-10 16:40:21Z pborelli $
  */
 
 /*
@@ -747,7 +747,11 @@ get_configure_dialog (GeditTimePlugin *plugin)
 	gchar *sf, *cf;
 	GtkWidget *error_widget;
 	gboolean ret;
-
+	gchar *root_objects[] = {
+		"time_dialog_content",
+		NULL
+	};
+	
 	gedit_debug (DEBUG_PLUGINS);
 
 	dialog = g_new0 (TimeConfigureDialog, 1);
@@ -771,18 +775,18 @@ get_configure_dialog (GeditTimePlugin *plugin)
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "time.glade2",
-					     "time_dialog_content",
-					     &error_widget,
-					     "time_dialog_content", &content,
-					     "formats_viewport", &viewport,
-					     "formats_tree", &dialog->list,
-					     "always_prompt", &dialog->prompt,
-					     "never_prompt", &dialog->use_list,
-					     "use_custom", &dialog->custom,
-					     "custom_entry", &dialog->custom_entry,
-					     "custom_format_example", &dialog->custom_format_example,
-					     NULL);
+	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "time.ui",
+					  root_objects,
+					  &error_widget,
+					  "time_dialog_content", &content,
+					  "formats_viewport", &viewport,
+					  "formats_tree", &dialog->list,
+					  "always_prompt", &dialog->prompt,
+					  "never_prompt", &dialog->use_list,
+					  "use_custom", &dialog->custom,
+					  "custom_entry", &dialog->custom_entry,
+					  "custom_format_example", &dialog->custom_format_example,
+					  NULL);
 
 	if (!ret)
 	{
@@ -841,6 +845,7 @@ get_configure_dialog (GeditTimePlugin *plugin)
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
 			    content, FALSE, FALSE, 0);
+	g_object_unref (content);
 	gtk_container_set_border_width (GTK_CONTAINER (content), 5);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
@@ -918,16 +923,16 @@ get_choose_format_dialog (GtkWindow                 *parent,
 
 	dialog = g_new0 (ChooseFormatDialog, 1);
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "time.glade2",
-					     "choose_format_dialog",
-					     &error_widget,
-					     "choose_format_dialog", &dialog->dialog,
-					     "choice_list", &dialog->list,
-					     "use_sel_format_radiobutton", &dialog->use_list,
-					     "use_custom_radiobutton", &dialog->custom,
-					     "custom_entry", &dialog->custom_entry,
-					     "custom_format_example", &dialog->custom_format_example,
-					     NULL);
+	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "time.ui",
+					  NULL,
+					  &error_widget,
+					  "choose_format_dialog", &dialog->dialog,
+					  "choice_list", &dialog->list,
+					  "use_sel_format_radiobutton", &dialog->use_list,
+					  "use_custom_radiobutton", &dialog->custom,
+					  "custom_entry", &dialog->custom_entry,
+					  "custom_format_example", &dialog->custom_format_example,
+					  NULL);
 
 	if (!ret)
 	{
@@ -1015,7 +1020,7 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 		{
 			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_HELP");
 			gedit_help_display (GTK_WINDOW (widget),
-					    "gedit.xml",
+					    NULL,
 					    "gedit-insert-date-time-plugin");
 			break;
 		}
@@ -1170,7 +1175,7 @@ configure_dialog_response_cb (GtkWidget           *widget,
 			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_HELP");
 
 			gedit_help_display (GTK_WINDOW (dialog),
-					    "gedit.xml",
+					    NULL,
 					    "gedit-date-time-configure");
 			break;
 		}
